@@ -42,10 +42,6 @@
 GST_DEBUG_CATEGORY_STATIC (gst_dtls_agent_debug);
 #define GST_CAT_DEFAULT gst_dtls_agent_debug
 
-G_DEFINE_TYPE (GstDtlsAgent, gst_dtls_agent, G_TYPE_OBJECT);
-
-#define GST_DTLS_AGENT_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GST_TYPE_DTLS_AGENT, GstDtlsAgentPrivate))
-
 enum
 {
   PROP_0,
@@ -62,6 +58,8 @@ struct _GstDtlsAgentPrivate
   GstDtlsCertificate *certificate;
 };
 
+G_DEFINE_TYPE_WITH_PRIVATE (GstDtlsAgent, gst_dtls_agent, G_TYPE_OBJECT);
+
 static void gst_dtls_agent_finalize (GObject * gobject);
 static void gst_dtls_agent_set_property (GObject *, guint prop_id,
     const GValue *, GParamSpec *);
@@ -122,13 +120,12 @@ _gst_dtls_init_openssl (void)
           OPENSSL_VERSION_TEXT);
       g_assert_not_reached ();
     }
-
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
     GST_INFO_OBJECT (NULL, "initializing openssl %lx", OPENSSL_VERSION_NUMBER);
     SSL_library_init ();
     SSL_load_error_strings ();
     ERR_load_BIO_strings ();
 
-#if OPENSSL_VERSION_NUMBER < 0x10100000L
     {
       gint i;
       gint num_locks;
@@ -151,8 +148,6 @@ gst_dtls_agent_class_init (GstDtlsAgentClass * klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
-  g_type_class_add_private (klass, sizeof (GstDtlsAgentPrivate));
-
   gobject_class->set_property = gst_dtls_agent_set_property;
   gobject_class->finalize = gst_dtls_agent_finalize;
 
@@ -179,7 +174,7 @@ ssl_warn_cb (const char *str, size_t len, void *u)
 static void
 gst_dtls_agent_init (GstDtlsAgent * self)
 {
-  GstDtlsAgentPrivate *priv = GST_DTLS_AGENT_GET_PRIVATE (self);
+  GstDtlsAgentPrivate *priv = gst_dtls_agent_get_instance_private (self);
   self->priv = priv;
 
   ERR_clear_error ();
@@ -189,24 +184,21 @@ gst_dtls_agent_init (GstDtlsAgent * self)
 #else
   priv->ssl_context = SSL_CTX_new (DTLSv1_method ());
 #endif
-  if (!priv->ssl_context) {
+  if (ERR_peek_error () || !priv->ssl_context) {
+    priv->ssl_context = NULL;
+
     GST_WARNING_OBJECT (self, "Error creating SSL Context");
     ERR_print_errors_cb (ssl_warn_cb, self);
 
     g_return_if_reached ();
   }
-  /* If any non-fatal issues happened, print them out and carry on */
-  if (ERR_peek_error ()) {
-    ERR_print_errors_cb (ssl_warn_cb, self);
-    ERR_clear_error ();
-  }
 
   SSL_CTX_set_verify_depth (priv->ssl_context, 2);
   SSL_CTX_set_tlsext_use_srtp (priv->ssl_context, "SRTP_AES128_CM_SHA1_80");
   SSL_CTX_set_cipher_list (priv->ssl_context,
       "ALL:!ADH:!LOW:!EXP:!MD5:@STRENGTH");
   SSL_CTX_set_read_ahead (priv->ssl_context, 1);
-#if OPENSSL_VERSION_NUMBER >= 0x1000200fL
+#if (OPENSSL_VERSION_NUMBER >= 0x1000200fL) && (OPENSSL_VERSION_NUMBER < 0x10100000L)
   SSL_CTX_set_ecdh_auto (priv->ssl_context, 1);
 #endif
 }
