@@ -45,13 +45,12 @@
 #ifndef MOTIONCELLS_H_
 #define MOTIONCELLS_H_
 
+#include <opencv2/core.hpp>
+#if (CV_MAJOR_VERSION >= 4)
+#include <opencv2/core/types_c.h>
 #include <opencv2/core/core_c.h>
-#ifdef HAVE_HIGHGUI_H
-#include <highgui.h>            // includes highGUI definitions
-#endif
-#ifdef HAVE_OPENCV2_HIGHGUI_HIGHGUI_C_H
-#include <opencv2/highgui/highgui_c.h>            // includes highGUI definitions
 #endif
+
 #include <fstream>
 #include <vector>
 #include <glib.h>
@@ -214,8 +213,10 @@ private:
 
   bool getIsNonZero (IplImage * img)
   {
-    for (int lin = 0; lin < img->height; lin++)
-      for (int col = 0; col < img->width; col++) {
+    int lin, col;
+
+    for (lin = 0; lin < img->height; lin++)
+      for (col = 0; col < img->width; col++) {
         if ((((uchar *) (img->imageData + img->widthStep * lin))[col]) > 0)
           return true;
       }
@@ -224,15 +225,17 @@ private:
 
   void setMotionCells (int p_frameWidth, int p_frameHeight)
   {
+    int i, j;
+
     m_cellwidth = (double) p_frameWidth / (double) m_gridx;
     m_cellheight = (double) p_frameHeight / (double) m_gridy;
     m_pCells = new Cell *[m_gridy];
-    for (int i = 0; i < m_gridy; i++)
+    for (i = 0; i < m_gridy; i++)
       m_pCells[i] = new Cell[m_gridx];
 
     //init cells
-    for (int i = 0; i < m_gridy; i++)
-      for (int j = 0; j < m_gridx; j++) {
+    for (i = 0; i < m_gridy; i++)
+      for (j = 0; j < m_gridx; j++) {
         m_pCells[i][j].MotionArea = 0;
         m_pCells[i][j].CellArea = 0;
         m_pCells[i][j].MotionPercent = 0;
@@ -242,7 +245,6 @@ private:
 
   IplImage *m_pcurFrame, *m_pprevFrame, *m_pdifferenceImage,
       *m_pbwImage,*transparencyimg;
-  CvSize m_frameSize;
   bool m_isVisible, m_changed_datafile, m_useAlpha, m_saveInDatafile;
   Cell **m_pCells;
   vector < MotionCellsIdx > m_MotionCells;
