@@ -61,7 +61,10 @@
 #endif
 
 #include "gstcvequalizehist.h"
+#include <opencv2/imgproc.hpp>
+#if (CV_MAJOR_VERSION >= 4)
 #include <opencv2/imgproc/imgproc_c.h>
+#endif
 
 GST_DEBUG_CATEGORY_STATIC (gst_cv_equalize_hist_debug);
 #define GST_CAT_DEFAULT gst_cv_equalize_hist_debug
