@@ -60,7 +60,10 @@
 
 #include "gst/opencv/gstopencvutils.h"
 #include "gstcvsmooth.h"
-#include <opencv2/imgproc/imgproc.hpp>
+#include <opencv2/imgproc.hpp>
+#if (CV_MAJOR_VERSION >= 4)
+#include <opencv2/imgproc/imgproc_c.h>
+#endif
 
 
 GST_DEBUG_CATEGORY_STATIC (gst_cv_smooth_debug);
@@ -371,11 +374,11 @@ gst_cv_smooth_transform_ip (GstOpencvVideoFilter * base, GstBuffer * buf,
     IplImage * img)
 {
   GstCvSmooth *filter = GST_CV_SMOOTH (base);
-  Mat mat = cvarrToMat(img);
+  Mat mat = cvarrToMat (img);
 
   if (filter->positionx != 0 || filter->positiony != 0 ||
       filter->width != G_MAXINT || filter->height != G_MAXINT) {
-    Size mat_size = mat.size();
+    Size mat_size = mat.size ();
 
     /* if the effect would start outside the image, just skip it */
     if (filter->positionx >= mat_size.width
@@ -385,12 +388,12 @@ gst_cv_smooth_transform_ip (GstOpencvVideoFilter * base, GstBuffer * buf,
     if (filter->width <= 0 || filter->height <= 0)
       return GST_FLOW_OK;
 
-    Rect mat_rect(filter->positionx,
+    Rect mat_rect (filter->positionx,
         filter->positiony,
-        MIN(filter->width, mat_size.width - filter->positionx),
-        MIN(filter->height, mat_size.height - filter->positiony));
+        MIN (filter->width, mat_size.width - filter->positionx),
+        MIN (filter->height, mat_size.height - filter->positiony));
 
-    mat = mat(mat_rect);
+    mat = mat (mat_rect);
   }
 
   switch (filter->type) {
