@@ -61,7 +61,10 @@
 #endif
 
 #include "gstedgedetect.h"
+#include <opencv2/imgproc.hpp>
+#if (CV_MAJOR_VERSION >= 4)
 #include <opencv2/imgproc/imgproc_c.h>
+#endif
 
 GST_DEBUG_CATEGORY_STATIC (gst_edge_detect_debug);
 #define GST_CAT_DEFAULT gst_edge_detect_debug
@@ -248,12 +251,14 @@ gst_edge_detect_set_caps (GstOpencvVideoFilter * transform,
   GstEdgeDetect *filter = GST_EDGE_DETECT (transform);
 
   if (filter->cvEdge != NULL) {
-      cvReleaseImage (&filter->cvGray);
-      cvReleaseImage (&filter->cvEdge);
+    cvReleaseImage (&filter->cvGray);
+    cvReleaseImage (&filter->cvEdge);
   }
 
-  filter->cvGray = cvCreateImage (cvSize (in_width, in_height), IPL_DEPTH_8U, 1);
-  filter->cvEdge = cvCreateImage (cvSize (in_width, in_height), IPL_DEPTH_8U, 1);
+  filter->cvGray =
+      cvCreateImage (cvSize (in_width, in_height), IPL_DEPTH_8U, 1);
+  filter->cvEdge =
+      cvCreateImage (cvSize (in_width, in_height), IPL_DEPTH_8U, 1);
 
   return TRUE;
 }
