@@ -48,10 +48,10 @@
 #define __GST_FACE_BLUR_H__
 
 #include <gst/gst.h>
-#include <opencv2/core/version.hpp>
-
 #include <gst/opencv/gstopencvvideofilter.h>
-#include <opencv2/objdetect/objdetect.hpp>
+
+#include <opencv2/core.hpp>
+#include <opencv2/objdetect.hpp>
 
 G_BEGIN_DECLS
 /* #defines don't like whitespacey bits */
