@@ -62,13 +62,16 @@
 
 /* element header */
 #include "gsthanddetect.h"
-#if (CV_MAJOR_VERSION >= 3)
 #include <opencv2/imgproc.hpp>
-#endif
+#if (CV_MAJOR_VERSION >= 4)
 #include <opencv2/imgproc/imgproc_c.h>
+#endif
 
 GST_DEBUG_CATEGORY_STATIC (gst_handdetect_debug);
 #define GST_CAT_DEFAULT gst_handdetect_debug
+#if (CV_MAJOR_VERSION < 4)
+#define CASCADE_DO_CANNY_PRUNING CV_HAAR_DO_CANNY_PRUNING
+#endif
 
 /* define HAAR files */
 #define HAAR_FILE_FIST GST_HAAR_CASCADES_DIR G_DIR_SEPARATOR_S "fist.xml"
@@ -194,21 +197,21 @@ gst_handdetect_class_init (GstHanddetectClass * klass)
       g_param_spec_boolean ("display",
           "Display",
           "Whether the detected hands are highlighted in output frame",
-          TRUE, (GParamFlags)G_PARAM_READWRITE)
+          TRUE, (GParamFlags) G_PARAM_READWRITE)
       );
   g_object_class_install_property (gobject_class,
       PROP_PROFILE_FIST,
       g_param_spec_string ("profile_fist",
           "Profile_fist",
           "Location of HAAR cascade file (fist gesture)",
-          HAAR_FILE_FIST, (GParamFlags)G_PARAM_READWRITE)
+          HAAR_FILE_FIST, (GParamFlags) G_PARAM_READWRITE)
       );
   g_object_class_install_property (gobject_class,
       PROP_PROFILE_PALM,
       g_param_spec_string ("profile_palm",
           "Profile_palm",
           "Location of HAAR cascade file (palm gesture)",
-          HAAR_FILE_PALM, (GParamFlags)G_PARAM_READWRITE)
+          HAAR_FILE_PALM, (GParamFlags) G_PARAM_READWRITE)
       );
   /* FIXME: property name needs fixing */
   g_object_class_install_property (gobject_class,
@@ -216,7 +219,7 @@ gst_handdetect_class_init (GstHanddetectClass * klass)
       g_param_spec_int ("ROI_X",
           "ROI_X",
           "X of left-top pointer in region of interest \nGestures in the defined region of interest will emit messages",
-          0, INT_MAX, 0, (GParamFlags)G_PARAM_READWRITE)
+          0, INT_MAX, 0, (GParamFlags) G_PARAM_READWRITE)
       );
   /* FIXME: property name needs fixing */
   g_object_class_install_property (gobject_class,
@@ -224,7 +227,7 @@ gst_handdetect_class_init (GstHanddetectClass * klass)
       g_param_spec_int ("ROI_Y",
           "ROI_Y",
           "Y of left-top pointer in region of interest \nGestures in the defined region of interest will emit messages",
-          0, INT_MAX, 0, (GParamFlags)G_PARAM_READWRITE)
+          0, INT_MAX, 0, (GParamFlags) G_PARAM_READWRITE)
       );
   /* FIXME: property name needs fixing */
   g_object_class_install_property (gobject_class,
@@ -232,7 +235,7 @@ gst_handdetect_class_init (GstHanddetectClass * klass)
       g_param_spec_int ("ROI_WIDTH",
           "ROI_WIDTH",
           "WIDTH of left-top pointer in region of interest \nGestures in the defined region of interest will emit messages",
-          0, INT_MAX, 0, (GParamFlags)G_PARAM_READWRITE)
+          0, INT_MAX, 0, (GParamFlags) G_PARAM_READWRITE)
       );
   /* FIXME: property name needs fixing */
   g_object_class_install_property (gobject_class,
@@ -240,7 +243,7 @@ gst_handdetect_class_init (GstHanddetectClass * klass)
       g_param_spec_int ("ROI_HEIGHT",
           "ROI_HEIGHT",
           "HEIGHT of left-top pointer in region of interest \nGestures in the defined region of interest will emit messages",
-          0, INT_MAX, 0, (GParamFlags)G_PARAM_READWRITE)
+          0, INT_MAX, 0, (GParamFlags) G_PARAM_READWRITE)
       );
 
   gst_element_class_set_static_metadata (element_class,
@@ -405,16 +408,16 @@ gst_handdetect_transform_ip (GstOpencvVideoFilter * transform,
 
   /* check detection cascades */
   if (filter->cvCascade_fist && filter->cvCascade_palm) {
-  /* cvt to gray colour space for hand detect */
+    /* cvt to gray colour space for hand detect */
     cvCvtColor (img, filter->cvGray, CV_RGB2GRAY);
 
     /* detect FIST gesture fist */
-    Mat image = cvarrToMat(filter->cvGray);
+    Mat image = cvarrToMat (filter->cvGray);
     Mat roi (image, Rect (filter->cvGray->origin,
             filter->cvGray->origin, filter->cvGray->width,
             filter->cvGray->height));
     filter->cvCascade_fist->detectMultiScale (roi, hands, 1.1, 2,
-        CV_HAAR_DO_CANNY_PRUNING, cvSize (24, 24), cvSize (0, 0));
+        CASCADE_DO_CANNY_PRUNING, cvSize (24, 24), cvSize (0, 0));
 
     /* if FIST gesture detected */
     if (!hands.empty ()) {
@@ -434,7 +437,7 @@ gst_handdetect_transform_ip (GstOpencvVideoFilter * transform,
       if (filter->prev_r == NULL)
         filter->prev_r = &temp_r;
       /* Get the best FIST gesture */
-      for (i = 0; i < hands.size(); i++) {
+      for (i = 0; i < hands.size (); i++) {
         r = &hands[i];
         distance = (int) sqrt (pow ((r->x - filter->prev_r->x),
                 2) + pow ((r->y - filter->prev_r->y), 2));
@@ -500,9 +503,14 @@ gst_handdetect_transform_ip (GstOpencvVideoFilter * transform,
         cvCircle (img, center, radius, CV_RGB (0, 0, 200), 1, 8, 0);
       }
     } else {
-     /* if NO FIST gesture, detecting PALM gesture */
+      /* if NO FIST gesture, detecting PALM gesture */
+#if (CV_MAJOR_VERSION >= 4)
+      filter->cvCascade_palm->detectMultiScale (roi, hands, 1.1, 2,
+          CASCADE_DO_CANNY_PRUNING, cvSize (24, 24), cvSize (0, 0));
+#else
       filter->cvCascade_palm->detectMultiScale (roi, hands, 1.1, 2,
           CV_HAAR_DO_CANNY_PRUNING, cvSize (24, 24), cvSize (0, 0));
+#endif
       /* if PALM detected */
       if (!hands.empty ()) {
         int min_distance, distance;
@@ -521,7 +529,7 @@ gst_handdetect_transform_ip (GstOpencvVideoFilter * transform,
         min_distance = img->width + img->height;
         /* Init filter->prev_r */
         temp_r = Rect (0, 0, 0, 0);
-       if (filter->prev_r == NULL)
+        if (filter->prev_r == NULL)
           filter->prev_r = &temp_r;
         /* Get the best PALM gesture */
         for (i = 0; i < hands.size (); ++i) {
