@@ -81,7 +81,7 @@
  * <refsect2>
  * <title>Example launch line</title>
  * |[
- * gst-launch-1.0  v4l2src device=/dev/video0 ! videoconvert ! video/x-raw,width=320,height=240 ! videoconvert ! segmentation test-mode=true method=2 ! videoconvert ! ximagesink
+ * gst-launch-1.0  v4l2src device=/dev/video0 ! videoconvert ! segmentation test-mode=true method=2 ! videoconvert ! ximagesink
  * ]|
  * </refsect2>
  */
@@ -91,12 +91,18 @@
 #endif
 
 #include "gstsegmentation.h"
-#include <opencv2/video/background_segm.hpp>
+#include <opencv2/imgproc.hpp>
+#if (CV_MAJOR_VERSION >= 4)
 #include <opencv2/imgproc/imgproc_c.h>
+#endif
 
 GST_DEBUG_CATEGORY_STATIC (gst_segmentation_debug);
 #define GST_CAT_DEFAULT gst_segmentation_debug
 
+using namespace cv;
+using namespace
+    cv::bgsegm;
+
 /* Filter signals and args */
 enum
 {
@@ -123,12 +129,17 @@ typedef enum
 #define DEFAULT_LEARNING_RATE  0.01
 
 #define GST_TYPE_SEGMENTATION_METHOD (gst_segmentation_method_get_type ())
-static GType
+static
+    GType
 gst_segmentation_method_get_type (void)
 {
-  static GType etype = 0;
+  static
+      GType
+      etype = 0;
   if (etype == 0) {
-    static const GEnumValue values[] = {
+    static const
+        GEnumValue
+        values[] = {
       {METHOD_BOOK, "Codebook-based segmentation (Bradski2008)", "codebook"},
       {METHOD_MOG, "Mixture-of-Gaussians segmentation (Bowden2001)", "mog"},
       {METHOD_MOG2, "Mixture-of-Gaussians segmentation (Zivkovic2004)", "mog2"},
@@ -139,55 +150,81 @@ gst_segmentation_method_get_type (void)
   return etype;
 }
 
-G_DEFINE_TYPE (GstSegmentation, gst_segmentation, GST_TYPE_VIDEO_FILTER);
-static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
+G_DEFINE_TYPE (GstSegmentation, gst_segmentation, GST_TYPE_OPENCV_VIDEO_FILTER);
+
+static
+    GstStaticPadTemplate
+    sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS (GST_VIDEO_CAPS_MAKE ("RGBA")));
 
-static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
+static
+    GstStaticPadTemplate
+    src_factory = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS (GST_VIDEO_CAPS_MAKE ("RGBA")));
 
 
-static void gst_segmentation_set_property (GObject * object, guint prop_id,
+static void
+gst_segmentation_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
-static void gst_segmentation_get_property (GObject * object, guint prop_id,
+static void
+gst_segmentation_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 
-static GstFlowReturn gst_segmentation_transform_ip (GstVideoFilter * btrans,
-    GstVideoFrame * frame);
+static
+    GstFlowReturn
+gst_segmentation_transform_ip (GstOpencvVideoFilter * filter,
+    GstBuffer * buffer, IplImage * img);
 
-static gboolean gst_segmentation_stop (GstBaseTransform * basesrc);
-static gboolean gst_segmentation_set_info (GstVideoFilter * filter,
-    GstCaps * incaps, GstVideoInfo * in_info,
-    GstCaps * outcaps, GstVideoInfo * out_info);
-static void gst_segmentation_release_all_pointers (GstSegmentation * filter);
+static
+    gboolean
+gst_segmentation_stop (GstBaseTransform * basesrc);
+static
+    gboolean
+gst_segmentation_set_caps (GstOpencvVideoFilter * filter, gint in_width,
+    gint in_height, gint in_depth, gint in_channels,
+    gint out_width, gint out_height, gint out_depth, gint out_channels);
+static void
+gst_segmentation_release_all_pointers (GstSegmentation * filter);
 
 /* Codebook algorithm + connected components functions*/
-static int update_codebook (unsigned char *p, codeBook * c,
+static int
+update_codebook (unsigned char *p, codeBook * c,
     unsigned *cbBounds, int numChannels);
-static int clear_stale_entries (codeBook * c);
-static unsigned char background_diff (unsigned char *p, codeBook * c,
+static int
+clear_stale_entries (codeBook * c);
+static unsigned char
+background_diff (unsigned char *p, codeBook * c,
     int numChannels, int *minMod, int *maxMod);
-static void find_connected_components (IplImage * mask, int poly1_hull0,
+static void
+find_connected_components (IplImage * mask, int poly1_hull0,
     float perimScale, CvMemStorage * mem_storage, CvSeq * contours);
 
 /* MOG (Mixture-of-Gaussians functions */
-static int initialise_mog (GstSegmentation * filter);
-static int run_mog_iteration (GstSegmentation * filter);
-static int run_mog2_iteration (GstSegmentation * filter);
-static int finalise_mog (GstSegmentation * filter);
+static int
+initialise_mog (GstSegmentation * filter);
+static int
+run_mog_iteration (GstSegmentation * filter);
+static int
+run_mog2_iteration (GstSegmentation * filter);
+static int
+finalise_mog (GstSegmentation * filter);
 
 /* initialize the segmentation's class */
 static void
 gst_segmentation_class_init (GstSegmentationClass * klass)
 {
-  GObjectClass *gobject_class;
-  GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
-  GstBaseTransformClass *basesrc_class = GST_BASE_TRANSFORM_CLASS (klass);
-  GstVideoFilterClass *video_class = (GstVideoFilterClass *) klass;
+  GObjectClass *
+      gobject_class;
+  GstElementClass *
+      element_class = GST_ELEMENT_CLASS (klass);
+  GstBaseTransformClass *
+      basesrc_class = GST_BASE_TRANSFORM_CLASS (klass);
+  GstOpencvVideoFilterClass *
+      cvfilter_class = (GstOpencvVideoFilterClass *) klass;
 
   gobject_class = (GObjectClass *) klass;
 
@@ -196,8 +233,8 @@ gst_segmentation_class_init (GstSegmentationClass * klass)
 
   basesrc_class->stop = gst_segmentation_stop;
 
-  video_class->transform_frame_ip = gst_segmentation_transform_ip;
-  video_class->set_info = gst_segmentation_set_info;
+  cvfilter_class->cv_trans_ip_func = gst_segmentation_transform_ip;
+  cvfilter_class->cv_set_caps = gst_segmentation_set_caps;
 
   g_object_class_install_property (gobject_class, PROP_METHOD,
       g_param_spec_enum ("method",
@@ -223,10 +260,8 @@ gst_segmentation_class_init (GstSegmentationClass * klass)
       "Create a Foregound/Background mask applying a particular algorithm",
       "Miguel Casas-Sanchez <miguelecasassanchez@gmail.com>");
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&src_factory));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&sink_factory));
+  gst_element_class_add_static_pad_template (element_class, &src_factory);
+  gst_element_class_add_static_pad_template (element_class, &sink_factory);
 
 }
 
@@ -242,15 +277,15 @@ gst_segmentation_init (GstSegmentation * filter)
   filter->test_mode = DEFAULT_TEST_MODE;
   filter->framecount = 0;
   filter->learning_rate = DEFAULT_LEARNING_RATE;
-  gst_base_transform_set_in_place (GST_BASE_TRANSFORM (filter), TRUE);
+  gst_opencv_video_filter_set_in_place (GST_OPENCV_VIDEO_FILTER (filter), TRUE);
 }
 
-
 static void
 gst_segmentation_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
 {
-  GstSegmentation *filter = GST_SEGMENTATION (object);
+  GstSegmentation *
+      filter = GST_SEGMENTATION (object);
 
   switch (prop_id) {
     case PROP_METHOD:
@@ -272,7 +307,8 @@ static void
 gst_segmentation_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec)
 {
-  GstSegmentation *filter = GST_SEGMENTATION (object);
+  GstSegmentation *
+      filter = GST_SEGMENTATION (object);
 
   switch (prop_id) {
     case PROP_METHOD:
@@ -290,25 +326,22 @@ gst_segmentation_get_property (GObject * object, guint prop_id,
   }
 }
 
-/* GstElement vmethod implementations */
-/* this function handles the link with other elements */
-static gboolean
-gst_segmentation_set_info (GstVideoFilter * filter,
-    GstCaps * incaps, GstVideoInfo * in_info,
-    GstCaps * outcaps, GstVideoInfo * out_info)
+static
+    gboolean
+gst_segmentation_set_caps (GstOpencvVideoFilter * filter, gint in_width,
+    gint in_height, gint in_depth, gint in_channels,
+    gint out_width, gint out_height, gint out_depth, gint out_channels)
 {
-  GstSegmentation *segmentation = GST_SEGMENTATION (filter);
+  GstSegmentation *
+      segmentation = GST_SEGMENTATION (filter);
   CvSize size;
 
-  size = cvSize (in_info->width, in_info->height);
-  segmentation->width = in_info->width;
-  segmentation->height = in_info->height;
-  /*  If cvRGB is already allocated, it means there's a cap modification, */
-  /*  so release first all the images. */
-  if (NULL != segmentation->cvRGBA)
-    gst_segmentation_release_all_pointers (segmentation);
+  size = cvSize (in_width, in_height);
+  segmentation->width = in_width;
+  segmentation->height = in_height;
 
-  segmentation->cvRGBA = cvCreateImageHeader (size, IPL_DEPTH_8U, 4);
+  if (NULL != segmentation->cvRGB)
+    gst_segmentation_release_all_pointers (segmentation);
 
   segmentation->cvRGB = cvCreateImage (size, IPL_DEPTH_8U, 3);
   segmentation->cvYUV = cvCreateImage (size, IPL_DEPTH_8U, 3);
@@ -337,12 +370,14 @@ gst_segmentation_set_info (GstVideoFilter * filter,
 }
 
 /* Clean up */
-static gboolean
+static
+    gboolean
 gst_segmentation_stop (GstBaseTransform * basesrc)
 {
-  GstSegmentation *filter = GST_SEGMENTATION (basesrc);
+  GstSegmentation *
+      filter = GST_SEGMENTATION (basesrc);
 
-  if (filter->cvRGBA != NULL)
+  if (filter->cvRGB != NULL)
     gst_segmentation_release_all_pointers (filter);
 
   return TRUE;
@@ -351,7 +386,6 @@ gst_segmentation_stop (GstBaseTransform * basesrc)
 static void
 gst_segmentation_release_all_pointers (GstSegmentation * filter)
 {
-  cvReleaseImage (&filter->cvRGBA);
   cvReleaseImage (&filter->cvRGB);
   cvReleaseImage (&filter->cvYUV);
   cvReleaseImage (&filter->cvFG);
@@ -365,19 +399,20 @@ gst_segmentation_release_all_pointers (GstSegmentation * filter)
   finalise_mog (filter);
 }
 
-static GstFlowReturn
-gst_segmentation_transform_ip (GstVideoFilter * btrans, GstVideoFrame * frame)
+static
+    GstFlowReturn
+gst_segmentation_transform_ip (GstOpencvVideoFilter * cvfilter,
+    GstBuffer * buffer, IplImage * img)
 {
-  GstSegmentation *filter = GST_SEGMENTATION (btrans);
-  int j;
+  GstSegmentation *
+      filter = GST_SEGMENTATION (cvfilter);
+  int
+      j;
 
-  /*  get image data from the input, which is RGBA */
-  filter->cvRGBA->imageData = (char *) GST_VIDEO_FRAME_COMP_DATA (frame, 0);
-  filter->cvRGBA->widthStep = GST_VIDEO_FRAME_COMP_STRIDE (frame, 0);
   filter->framecount++;
 
   /*  Image preprocessing: color space conversion etc */
-  cvCvtColor (filter->cvRGBA, filter->cvRGB, CV_RGBA2RGB);
+  cvCvtColor (img, filter->cvRGB, CV_RGBA2RGB);
   cvCvtColor (filter->cvRGB, filter->cvYUV, CV_RGB2YCrCb);
 
   /* Create and update a fg/bg model using a codebook approach following the
@@ -388,9 +423,12 @@ gst_segmentation_transform_ip (GstVideoFilter * btrans, GstVideoFrame * frame)
    * [2] "Real-time Foreground-Background Segmentation using Codebook Model",
    * Real-time Imaging, Volume 11, Issue 3, Pages 167-256, June 2005. */
   if (METHOD_BOOK == filter->method) {
-    unsigned cbBounds[3] = { 10, 5, 5 };
-    int minMod[3] = { 20, 20, 20 }, maxMod[3] = {
-    20, 20, 20};
+    unsigned
+    cbBounds[3] = { 10, 5, 5 };
+    int
+    minMod[3] = { 20, 20, 20 }, maxMod[3] = {
+      20, 20, 20
+    };
 
     if (filter->framecount < 30) {
       /* Learning background phase: update_codebook on every frame */
@@ -415,7 +453,7 @@ gst_segmentation_transform_ip (GstVideoFilter * btrans, GstVideoFrame * frame)
         if (background_diff
             ((uchar *) filter->cvYUV->imageData + j * 3,
                 (codeBook *) & (filter->TcodeBook[j]), 3, minMod, maxMod)) {
-          filter->cvFG->imageData[j] = 255;
+          filter->cvFG->imageData[j] = (char) 255;
         } else {
           filter->cvFG->imageData[j] = 0;
         }
@@ -459,10 +497,10 @@ gst_segmentation_transform_ip (GstVideoFilter * btrans, GstVideoFrame * frame)
 
     cvSplit (filter->cvRGB, filter->ch1, filter->ch2, filter->ch3, NULL);
   } else
-    cvSplit (filter->cvRGBA, filter->ch1, filter->ch2, filter->ch3, NULL);
+    cvSplit (img, filter->ch1, filter->ch2, filter->ch3, NULL);
 
   /*  copy anyhow the fg/bg to the alpha channel in the output image */
-  cvMerge (filter->ch1, filter->ch2, filter->ch3, filter->cvFG, filter->cvRGBA);
+  cvMerge (filter->ch1, filter->ch2, filter->ch3, filter->cvFG, img);
 
 
   return GST_FLOW_OK;
@@ -505,9 +543,14 @@ update_codebook (unsigned char *p, codeBook * c, unsigned *cbBounds,
     int numChannels)
 {
 /* c->t+=1; */
-  unsigned int high[3], low[3];
-  int n, i;
-  int matchChannel;
+  unsigned int
+      high[3],
+      low[3];
+  int
+      n,
+      i;
+  int
+      matchChannel;
 
   for (n = 0; n < numChannels; n++) {
     high[n] = p[n] + cbBounds[n];
@@ -546,13 +589,15 @@ update_codebook (unsigned char *p, codeBook * c, unsigned *cbBounds,
 /*  OVERHEAD TO TRACK POTENTIAL STALE ENTRIES */
   for (int s = 0; s < c->numEntries; s++) {
 /*  Track which codebook entries are going stale: */
-    int negRun = c->t - c->cb[s]->t_last_update;
+    int
+        negRun = c->t - c->cb[s]->t_last_update;
     if (c->cb[s]->stale < negRun)
       c->cb[s]->stale = negRun;
   }
 /*  ENTER A NEW CODEWORD IF NEEDED */
   if (i == c->numEntries) {     /* if no existing codeword found, make one */
-    code_element **foo =
+    code_element **
+        foo =
         (code_element **) g_malloc (sizeof (code_element *) *
         (c->numEntries + 1));
     for (int ii = 0; ii < c->numEntries; ii++) {
@@ -599,12 +644,18 @@ update_codebook (unsigned char *p, codeBook * c, unsigned *cbBounds,
 int
 clear_stale_entries (codeBook * c)
 {
-  int staleThresh = c->t >> 1;
-  int *keep = (int *) g_malloc (sizeof (int) * (c->numEntries));
-  int keepCnt = 0;
-  code_element **foo;
-  int k;
-  int numCleared;
+  int
+      staleThresh = c->t >> 1;
+  int *
+      keep = (int *) g_malloc (sizeof (int) * (c->numEntries));
+  int
+      keepCnt = 0;
+  code_element **
+      foo;
+  int
+      k;
+  int
+      numCleared;
 /*  SEE WHICH CODEBOOK ENTRIES ARE TOO STALE */
   for (int i = 0; i < c->numEntries; i++) {
     if (c->cb[i]->stale > staleThresh)
@@ -664,9 +715,11 @@ unsigned char
 background_diff (unsigned char *p, codeBook * c, int numChannels,
     int *minMod, int *maxMod)
 {
-  int matchChannel;
+  int
+      matchChannel;
 /*  SEE IF THIS FITS AN EXISTING CODEWORD */
-  int i;
+  int
+      i;
   for (i = 0; i < c->numEntries; i++) {
     matchChannel = 0;
     for (int n = 0; n < numChannels; n++) {
@@ -722,11 +775,17 @@ find_connected_components (IplImage * mask, int poly1_hull0, float perimScale,
     CvMemStorage * mem_storage, CvSeq * contours)
 {
   CvContourScanner scanner;
-  CvSeq *c;
-  int numCont = 0;
+  CvSeq *
+      c;
+  int
+      numCont = 0;
   /* Just some convenience variables */
-  const CvScalar CVX_WHITE = CV_RGB (0xff, 0xff, 0xff);
-  const CvScalar CVX_BLACK = CV_RGB (0x00, 0x00, 0x00);
+  const
+      CvScalar
+      CVX_WHITE = CV_RGB (0xff, 0xff, 0xff);
+  const
+      CvScalar
+      CVX_BLACK = CV_RGB (0x00, 0x00, 0x00);
 
   /* CLEAN UP RAW MASK */
   cvMorphologyEx (mask, mask, 0, 0, CV_MOP_OPEN, CVCLOSE_ITR);
@@ -742,15 +801,18 @@ find_connected_components (IplImage * mask, int poly1_hull0, float perimScale,
       CV_RETR_TREE, CV_CHAIN_APPROX_SIMPLE, cvPoint (0, 0));
 
   while ((c = cvFindNextContour (scanner)) != NULL) {
-    double len = cvContourArea (c, CV_WHOLE_SEQ, 0);
+    double
+        len = cvContourArea (c, CV_WHOLE_SEQ, 0);
     /* calculate perimeter len threshold: */
-    double q = (mask->height + mask->width) / perimScale;
+    double
+        q = (mask->height + mask->width) / perimScale;
     /* Get rid of blob if its perimeter is too small: */
     if (len < q) {
       cvSubstituteContour (scanner, NULL);
     } else {
       /* Smooth its edges if its large enough */
-      CvSeq *c_new;
+      CvSeq *
+          c_new;
       if (poly1_hull0) {
         /* Polygonal approximation */
         c_new =
@@ -779,11 +841,12 @@ find_connected_components (IplImage * mask, int poly1_hull0, float perimScale,
 int
 initialise_mog (GstSegmentation * filter)
 {
-  filter->img_input_as_cvMat = (void *) new cv::Mat (filter->cvYUV, false);
-  filter->img_fg_as_cvMat = (void *) new cv::Mat (filter->cvFG, false);
+  filter->img_input_as_cvMat = (void *) new
+      Mat (cvarrToMat (filter->cvYUV, false));
+  filter->img_fg_as_cvMat = (void *) new Mat (cvarrToMat (filter->cvFG, false));
 
-  filter->mog = (void *) new cv::BackgroundSubtractorMOG ();
-  filter->mog2 = (void *) new cv::BackgroundSubtractorMOG2 ();
+  filter->mog = bgsegm::createBackgroundSubtractorMOG ();
+  filter->mog2 = createBackgroundSubtractorMOG2 ();
 
   return (0);
 }
@@ -806,9 +869,8 @@ run_mog_iteration (GstSegmentation * filter)
      European Workshop on Advanced Video-Based Surveillance Systems, 2001
    */
 
-  (*((cv::BackgroundSubtractorMOG *) filter->mog)) (*((cv::Mat *) filter->
-          img_input_as_cvMat), *((cv::Mat *) filter->img_fg_as_cvMat),
-      filter->learning_rate);
+  filter->mog->apply (*((Mat *) filter->img_input_as_cvMat),
+      *((Mat *) filter->img_fg_as_cvMat), filter->learning_rate);
 
   return (0);
 }
@@ -816,10 +878,9 @@ run_mog_iteration (GstSegmentation * filter)
 int
 run_mog2_iteration (GstSegmentation * filter)
 {
-  ((cv::Mat *) filter->img_input_as_cvMat)->data =
+  ((Mat *) filter->img_input_as_cvMat)->data =
       (uchar *) filter->cvYUV->imageData;
-  ((cv::Mat *) filter->img_fg_as_cvMat)->data =
-      (uchar *) filter->cvFG->imageData;
+  ((Mat *) filter->img_fg_as_cvMat)->data = (uchar *) filter->cvFG->imageData;
 
   /*
      BackgroundSubtractorMOG2 [1], Gaussian Mixture-based Background/Foreground
@@ -834,9 +895,8 @@ run_mog2_iteration (GstSegmentation * filter)
      Letters, vol. 27, no. 7, pages 773-780, 2006.
    */
 
-  (*((cv::BackgroundSubtractorMOG *) filter->mog2)) (*((cv::Mat *) filter->
-          img_input_as_cvMat), *((cv::Mat *) filter->img_fg_as_cvMat),
-      filter->learning_rate);
+  filter->mog2->apply (*((Mat *) filter->img_input_as_cvMat),
+      *((Mat *) filter->img_fg_as_cvMat), filter->learning_rate);
 
   return (0);
 }
@@ -844,9 +904,11 @@ run_mog2_iteration (GstSegmentation * filter)
 int
 finalise_mog (GstSegmentation * filter)
 {
-  delete (cv::Mat *) filter->img_input_as_cvMat;
-  delete (cv::Mat *) filter->img_fg_as_cvMat;
-  delete (cv::BackgroundSubtractorMOG *) filter->mog;
-  delete (cv::BackgroundSubtractorMOG2 *) filter->mog2;
+  delete (Mat *) filter->img_input_as_cvMat;
+  delete (Mat *) filter->img_fg_as_cvMat;
+
+  filter->mog.release ();
+  filter->mog2.release ();
+
   return (0);
 }
