@@ -47,10 +47,12 @@
 #define __GST_OPENCV_TEXT_OVERLAY_H__
 
 #include <gst/opencv/gstopencvvideofilter.h>
-#include <opencv2/core/core_c.h>
-#if (CV_MAJOR_VERSION >= 3)
+#include <opencv2/core.hpp>
+#include <opencv2/imgproc.hpp>
+#if (CV_MAJOR_VERSION >= 4)
 #include <opencv2/imgproc/imgproc_c.h>
 #endif
+
 G_BEGIN_DECLS
 
 /* #defines don't like whitespacey bits */
