@@ -100,6 +100,10 @@
  * the clients will start with a rollover counter of 0 which will
  * probably be incorrect if the stream has been transmitted for a
  * while to other clients.
+ *
+ * This element supports sending with a single Master Key, it is possible to set the
+ * Master Key Identifier (MKI) using the "mki" property. If this property is set, the MKI
+ * will be added to every buffer.
  */
 
 #include "gstsrtpenc.h"
@@ -152,7 +156,8 @@ enum
   PROP_RANDOM_KEY,
   PROP_REPLAY_WINDOW_SIZE,
   PROP_ALLOW_REPEAT_TX,
-  PROP_STATS
+  PROP_STATS,
+  PROP_MKI
 };
 
 typedef struct ProcessBufferItData
@@ -160,6 +165,7 @@ typedef struct ProcessBufferItData
   GstSrtpEnc *filter;
   GstPad *pad;
   GstBufferList *out_list;
+  GstFlowReturn flowret;
   gboolean is_rtcp;
 } ProcessBufferItData;
 
@@ -317,6 +323,13 @@ gst_srtp_enc_class_init (GstSrtpEncClass * klass)
   g_object_class_install_property (gobject_class, PROP_STATS,
       g_param_spec_boxed ("stats", "Statistics", "Various statistics",
           GST_TYPE_STRUCTURE, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+#ifdef HAVE_SRTP2
+  g_object_class_install_property (gobject_class, PROP_MKI,
+      g_param_spec_boxed ("mki", "MKI",
+          "Master key Identifier (NULL means no MKI)", GST_TYPE_BUFFER,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS |
+          GST_PARAM_MUTABLE_PLAYING));
+#endif
 
   /**
    * GstSrtpEnc::soft-limit:
@@ -371,6 +384,12 @@ gst_srtp_enc_create_session (GstSrtpEnc * filter)
   srtp_policy_t policy;
   GstMapInfo map;
   guchar tmp[1];
+#ifdef HAVE_SRTP2
+  srtp_master_key_t mkey;
+  srtp_master_key_t *mkey_ptr = &mkey;
+  gboolean has_mki = FALSE;
+  GstMapInfo mki_map;
+#endif
 
   memset (&policy, 0, sizeof (srtp_policy_t));
 
@@ -415,6 +434,29 @@ gst_srtp_enc_create_session (GstSrtpEnc * filter)
     policy.key = tmp;
   }
 
+#ifdef HAVE_SRTP2
+  if (filter->mki) {
+    if (!gst_buffer_map (filter->mki, &mki_map, GST_MAP_READ)) {
+      GST_OBJECT_UNLOCK (filter);
+      GST_ELEMENT_ERROR (filter, LIBRARY, SETTINGS, ("Could not map MKI"),
+          (NULL));
+      GST_OBJECT_LOCK (filter);
+
+      ret = srtp_err_status_fail;
+      goto done;
+    }
+    has_mki = TRUE;
+
+    policy.num_master_keys = 1;
+    policy.keys = &mkey_ptr;
+    mkey.key = policy.key;
+    policy.key = NULL;
+
+    mkey.mki_id = (guchar *) mki_map.data;
+    mkey.mki_size = mki_map.size;
+  }
+#endif
+
   policy.ssrc.value = 0;
   policy.ssrc.type = ssrc_any_outbound;
   policy.next = NULL;
@@ -428,9 +470,17 @@ gst_srtp_enc_create_session (GstSrtpEnc * filter)
   ret = srtp_create (&filter->session, &policy);
   filter->first_session = FALSE;
 
+#ifdef HAVE_SRTP2
+done:
+
+  if (has_mki)
+    gst_buffer_unmap (filter->mki, &mki_map);
+#endif
+
   if (HAS_CRYPTO (filter))
     gst_buffer_unmap (filter->key, &map);
 
+
   return ret;
 }
 
@@ -594,9 +644,8 @@ gst_srtp_enc_dispose (GObject * object)
   }
   gst_iterator_free (it);
 
-  if (filter->key)
-    gst_buffer_unref (filter->key);
-  filter->key = NULL;
+  gst_buffer_replace (&filter->key, NULL);
+  gst_buffer_replace (&filter->mki, NULL);
 
   if (filter->ssrcs_set)
     g_hash_table_unref (filter->ssrcs_set);
@@ -657,8 +706,7 @@ gst_srtp_enc_set_property (GObject * object, guint prop_id,
 
   switch (prop_id) {
     case PROP_MKEY:
-      if (filter->key)
-        gst_buffer_unref (filter->key);
+      gst_clear_buffer (&filter->key);
       filter->key = g_value_dup_boxed (value);
       filter->key_changed = TRUE;
       GST_INFO_OBJECT (object, "Set property: key=[%p]", filter->key);
@@ -696,7 +744,14 @@ gst_srtp_enc_set_property (GObject * object, guint prop_id,
     case PROP_ALLOW_REPEAT_TX:
       filter->allow_repeat_tx = g_value_get_boolean (value);
       break;
-
+#ifdef HAVE_SRTP2
+    case PROP_MKI:
+      gst_clear_buffer (&filter->mki);
+      filter->mki = g_value_dup_boxed (value);
+      filter->key_changed = TRUE;
+      GST_INFO_OBJECT (object, "Set property: mki=[%p]", filter->mki);
+      break;
+#endif
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -741,6 +796,12 @@ gst_srtp_enc_get_property (GObject * object, guint prop_id,
     case PROP_STATS:
       g_value_take_boxed (value, gst_srtp_enc_create_stats (filter));
       break;
+#ifdef HAVE_SRTP2
+    case PROP_MKI:
+      if (filter->mki)
+        g_value_set_boxed (value, filter->mki);
+      break;
+#endif
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -815,6 +876,11 @@ gst_srtp_enc_sink_setcaps (GstPad * pad, GstSrtpEnc * filter,
   if (HAS_CRYPTO (filter))
     gst_structure_set (ps, "srtp-key", GST_TYPE_BUFFER, filter->key, NULL);
 
+#ifdef HAVE_SRTP2
+  if (filter->mki)
+    gst_structure_set (ps, "mki", GST_TYPE_BUFFER, filter->mki, NULL);
+#endif
+
   /* Add srtp-specific params to source caps */
   gst_structure_set (ps,
       "srtp-cipher", G_TYPE_STRING,
@@ -893,7 +959,7 @@ gst_srtp_enc_sink_query (GstPad * pad, GstObject * parent, GstQuery * query,
         else
           gst_structure_set_name (ps, "application/x-rtp");
         gst_structure_remove_fields (ps, "srtp-key", "srtp-cipher", "srtp-auth",
-            "srtcp-cipher", "srtcp-auth", NULL);
+            "srtcp-cipher", "srtcp-auth", "mki", NULL);
       }
 
       gst_query_set_caps_result (query, ret);
@@ -1029,10 +1095,11 @@ gst_srtp_enc_check_set_caps (GstSrtpEnc * filter, GstPad * pad,
   return GST_FLOW_OK;
 }
 
-static GstBuffer *
+static GstFlowReturn
 gst_srtp_enc_process_buffer (GstSrtpEnc * filter, GstPad * pad,
-    GstBuffer * buf, gboolean is_rtcp)
+    GstBuffer * buf, gboolean is_rtcp, GstBuffer ** outbuf_ptr)
 {
+  GstFlowReturn ret = GST_FLOW_OK;
   gint size_max, size;
   GstBuffer *bufout = NULL;
   GstMapInfo mapout;
@@ -1051,10 +1118,25 @@ gst_srtp_enc_process_buffer (GstSrtpEnc * filter, GstPad * pad,
 
   gst_srtp_init_event_reporter ();
 
+  if (filter->session == NULL) {
+    /* The rtcp session disappeared (element shutting down) */
+    GST_OBJECT_UNLOCK (filter);
+    ret = GST_FLOW_FLUSHING;
+    goto fail;
+  }
+#ifdef HAVE_SRTP2
+  if (is_rtcp)
+    err = srtp_protect_rtcp_mki (filter->session, mapout.data, &size,
+        (filter->mki != NULL), 0);
+  else
+    err = srtp_protect_mki (filter->session, mapout.data, &size,
+        (filter->mki != NULL), 0);
+#else
   if (is_rtcp)
     err = srtp_protect_rtcp (filter->session, mapout.data, &size);
   else
     err = srtp_protect (filter->session, mapout.data, &size);
+#endif
 
   GST_OBJECT_UNLOCK (filter);
 
@@ -1073,20 +1155,23 @@ gst_srtp_enc_process_buffer (GstSrtpEnc * filter, GstPad * pad,
     GST_ELEMENT_ERROR (GST_ELEMENT_CAST (filter), STREAM, ENCODE,
         ("Key usage limit has been reached"),
         ("Unable to protect buffer (hard key usage limit reached)"));
+    ret = GST_FLOW_ERROR;
     goto fail;
 
   } else {
     /* srtp_protect failed */
     GST_ELEMENT_ERROR (filter, LIBRARY, FAILED, (NULL),
         ("Unable to protect buffer (protect failed) code %d", err));
+    ret = GST_FLOW_ERROR;
     goto fail;
   }
 
-  return bufout;
+  *outbuf_ptr = bufout;
+  return ret;
 
 fail:
   gst_buffer_unref (bufout);
-  return NULL;
+  return ret;
 }
 
 static GstFlowReturn
@@ -1112,17 +1197,17 @@ gst_srtp_enc_chain (GstPad * pad, GstObject * parent, GstBuffer * buf,
 
   GST_OBJECT_UNLOCK (filter);
 
-  if ((bufout = gst_srtp_enc_process_buffer (filter, pad, buf, is_rtcp))) {
-    /* Push buffer to source pad */
-    otherpad = get_rtp_other_pad (pad);
-    ret = gst_pad_push (otherpad, bufout);
-    bufout = NULL;
+  ret = gst_srtp_enc_process_buffer (filter, pad, buf, is_rtcp, &bufout);
+  if (ret != GST_FLOW_OK)
+    goto out;
 
-    if (ret != GST_FLOW_OK)
-      goto out;
-  } else {
-    goto fail;
-  }
+  /* Push buffer to source pad */
+  otherpad = get_rtp_other_pad (pad);
+  ret = gst_pad_push (otherpad, bufout);
+  bufout = NULL;
+
+  if (ret != GST_FLOW_OK)
+    goto out;
 
   GST_OBJECT_LOCK (filter);
 
@@ -1137,14 +1222,8 @@ gst_srtp_enc_chain (GstPad * pad, GstObject * parent, GstBuffer * buf,
   GST_OBJECT_UNLOCK (filter);
 
 out:
-
   gst_buffer_unref (buf);
-
   return ret;
-
-fail:
-  ret = GST_FLOW_ERROR;
-  goto out;
 }
 
 static gboolean
@@ -1152,15 +1231,17 @@ process_buffer_it (GstBuffer ** buffer, guint index, gpointer user_data)
 {
   ProcessBufferItData *data = user_data;
   GstBuffer *bufout;
+  GstFlowReturn ret;
 
-  if ((bufout =
-          gst_srtp_enc_process_buffer (data->filter, data->pad, *buffer,
-              data->is_rtcp))) {
-    gst_buffer_list_add (data->out_list, bufout);
-  } else {
-    GST_WARNING_OBJECT (data->filter, "Error encoding buffer, dropping");
+  ret = gst_srtp_enc_process_buffer (data->filter, data->pad, *buffer,
+      data->is_rtcp, &bufout);
+  if (ret != GST_FLOW_OK) {
+    data->flowret = ret;
+    return FALSE;
   }
 
+  gst_buffer_list_add (data->out_list, bufout);
+
   return TRUE;
 }
 
@@ -1199,8 +1280,12 @@ gst_srtp_enc_chain_list (GstPad * pad, GstObject * parent,
   process_data.pad = pad;
   process_data.is_rtcp = is_rtcp;
   process_data.out_list = out_list;
+  process_data.flowret = GST_FLOW_OK;
 
-  gst_buffer_list_foreach (buf_list, process_buffer_it, &process_data);
+  if (!gst_buffer_list_foreach (buf_list, process_buffer_it, &process_data)) {
+    ret = process_data.flowret;
+    goto out;
+  }
 
   if (!gst_buffer_list_length (out_list)) {
     gst_buffer_list_unref (out_list);
