@@ -54,14 +54,6 @@ enum
 
 static guint gst_vulkan_instance_signals[LAST_SIGNAL] = { 0 };
 
-#define gst_vulkan_instance_parent_class parent_class
-G_DEFINE_TYPE_WITH_CODE (GstVulkanInstance, gst_vulkan_instance,
-    GST_TYPE_OBJECT, GST_DEBUG_CATEGORY_INIT (GST_CAT_DEFAULT,
-        "vulkaninstance", 0, "Vulkan Instance");
-    GST_DEBUG_CATEGORY_INIT (GST_VULKAN_DEBUG_CAT,
-        "vulkandebug", 0, "Vulkan Debug");
-    GST_DEBUG_CATEGORY_GET (GST_CAT_CONTEXT, "GST_CONTEXT"));
-
 static void gst_vulkan_instance_finalize (GObject * object);
 
 struct _GstVulkanInstancePrivate
@@ -69,6 +61,15 @@ struct _GstVulkanInstancePrivate
   gboolean opened;
 };
 
+#define gst_vulkan_instance_parent_class parent_class
+G_DEFINE_TYPE_WITH_CODE (GstVulkanInstance, gst_vulkan_instance,
+    GST_TYPE_OBJECT, G_ADD_PRIVATE (GstVulkanInstance)
+    GST_DEBUG_CATEGORY_INIT (GST_CAT_DEFAULT,
+        "vulkaninstance", 0, "Vulkan Instance");
+    GST_DEBUG_CATEGORY_INIT (GST_VULKAN_DEBUG_CAT,
+        "vulkandebug", 0, "Vulkan Debug");
+    GST_DEBUG_CATEGORY_GET (GST_CAT_CONTEXT, "GST_CONTEXT"));
+
 GstVulkanInstance *
 gst_vulkan_instance_new (void)
 {
@@ -83,8 +84,7 @@ gst_vulkan_instance_new (void)
 static void
 gst_vulkan_instance_init (GstVulkanInstance * instance)
 {
-  instance->priv = G_TYPE_INSTANCE_GET_PRIVATE ((instance),
-      GST_TYPE_VULKAN_INSTANCE, GstVulkanInstancePrivate);
+  instance->priv = gst_vulkan_instance_get_instance_private (instance);
 }
 
 static void
@@ -94,8 +94,6 @@ gst_vulkan_instance_class_init (GstVulkanInstanceClass * klass)
   gst_vulkan_image_memory_init_once ();
   gst_vulkan_buffer_memory_init_once ();
 
-  g_type_class_add_private (klass, sizeof (GstVulkanInstancePrivate));
-
   /**
    * GstVulkanInstance::create-device:
    * @object: the #GstVulkanDisplay
@@ -182,6 +180,8 @@ gst_vulkan_instance_open (GstVulkanInstance * instance, GError ** error)
   gchar **enabled_layers;
   VkResult err;
 
+  g_return_val_if_fail (GST_IS_VULKAN_INSTANCE (instance), FALSE);
+
   GST_OBJECT_LOCK (instance);
   if (instance->priv->opened) {
     GST_OBJECT_UNLOCK (instance);
