@@ -36,6 +36,7 @@
 #endif
 
 #include <gst/gst.h>
+#include <gst/codecparsers/codecparsers-prelude.h>
 
 G_BEGIN_DECLS
 
@@ -77,9 +78,12 @@ G_BEGIN_DECLS
  * @GST_H264_PROFILE_BASELINE: Baseline profile (A.2.1)
  * @GST_H264_PROFILE_MAIN: Main profile (A.2.2)
  * @GST_H264_PROFILE_EXTENDED: Extended profile (A.2.3)
- * @GST_H264_PROFILE_HIGH: High profile (A.2.4)
+ * @GST_H264_PROFILE_HIGH: High profile (A.2.4),
+ * or Progressive High profile (A.2.4.1), or Constrained High profile (A.2.4.2)
+ * depending on constraint_set4_flag and constraint_set5_flag
  * @GST_H264_PROFILE_HIGH10: High 10 profile (A.2.5) or High 10 Intra
- *   profile (A.2.8), depending on constraint_set3_flag
+ *   profile (A.2.8), or Progressive High 10 profile (A.2.5.1) depending on
+ *   constraint_set3_flag and constraint_set4_flag
  * @GST_H264_PROFILE_HIGH_422: High 4:2:2 profile (A.2.6) or High
  *   4:2:2 Intra profile (A.2.9), depending on constraint_set3_flag
  * @GST_H264_PROFILE_HIGH_444: High 4:4:4 Predictive profile (A.2.7)
@@ -1024,87 +1028,87 @@ struct _GstH264NalParser
   GstH264PPS *last_pps;
 };
 
-GST_EXPORT
+GST_CODEC_PARSERS_API
 GstH264NalParser *gst_h264_nal_parser_new             (void);
 
-GST_EXPORT
+GST_CODEC_PARSERS_API
 GstH264ParserResult gst_h264_parser_identify_nalu     (GstH264NalParser *nalparser,
                                                        const guint8 *data, guint offset,
                                                        gsize size, GstH264NalUnit *nalu);
 
-GST_EXPORT
+GST_CODEC_PARSERS_API
 GstH264ParserResult gst_h264_parser_identify_nalu_unchecked (GstH264NalParser *nalparser,
                                                        const guint8 *data, guint offset,
                                                        gsize size, GstH264NalUnit *nalu);
 
-GST_EXPORT
+GST_CODEC_PARSERS_API
 GstH264ParserResult gst_h264_parser_identify_nalu_avc (GstH264NalParser *nalparser, const guint8 *data,
                                                        guint offset, gsize size, guint8 nal_length_size,
                                                        GstH264NalUnit *nalu);
 
-GST_EXPORT
+GST_CODEC_PARSERS_API
 GstH264ParserResult gst_h264_parser_parse_nal         (GstH264NalParser *nalparser,
                                                        GstH264NalUnit *nalu);
 
-GST_EXPORT
+GST_CODEC_PARSERS_API
 GstH264ParserResult gst_h264_parser_parse_slice_hdr   (GstH264NalParser *nalparser, GstH264NalUnit *nalu,
                                                        GstH264SliceHdr *slice, gboolean parse_pred_weight_table,
                                                        gboolean parse_dec_ref_pic_marking);
 
-GST_EXPORT
+GST_CODEC_PARSERS_API
 GstH264ParserResult gst_h264_parser_parse_subset_sps  (GstH264NalParser *nalparser, GstH264NalUnit *nalu,
                                                        GstH264SPS *sps, gboolean parse_vui_params);
 
-GST_EXPORT
+GST_CODEC_PARSERS_API
 GstH264ParserResult gst_h264_parser_parse_sps         (GstH264NalParser *nalparser, GstH264NalUnit *nalu,
                                                        GstH264SPS *sps, gboolean parse_vui_params);
 
-GST_EXPORT
+GST_CODEC_PARSERS_API
 GstH264ParserResult gst_h264_parser_parse_pps         (GstH264NalParser *nalparser,
                                                        GstH264NalUnit *nalu, GstH264PPS *pps);
 
-GST_EXPORT
+GST_CODEC_PARSERS_API
 GstH264ParserResult gst_h264_parser_parse_sei         (GstH264NalParser *nalparser,
                                                        GstH264NalUnit *nalu, GArray ** messages);
 
-GST_EXPORT
+GST_CODEC_PARSERS_API
 void gst_h264_nal_parser_free                         (GstH264NalParser *nalparser);
 
-GST_EXPORT
+GST_CODEC_PARSERS_API
 GstH264ParserResult gst_h264_parse_subset_sps         (GstH264NalUnit *nalu,
                                                        GstH264SPS *sps, gboolean parse_vui_params);
 
-GST_EXPORT
+GST_CODEC_PARSERS_API
 GstH264ParserResult gst_h264_parse_sps                (GstH264NalUnit *nalu,
                                                        GstH264SPS *sps, gboolean parse_vui_params);
 
-GST_EXPORT
+GST_CODEC_PARSERS_API
 GstH264ParserResult gst_h264_parse_pps                (GstH264NalParser *nalparser,
                                                        GstH264NalUnit *nalu, GstH264PPS *pps);
 
-GST_EXPORT
+GST_CODEC_PARSERS_API
 void                gst_h264_sps_clear                (GstH264SPS *sps);
 
-GST_EXPORT
+GST_CODEC_PARSERS_API
 void                gst_h264_pps_clear                (GstH264PPS *pps);
 
-GST_EXPORT
+GST_CODEC_PARSERS_API
 void    gst_h264_quant_matrix_8x8_get_zigzag_from_raster (guint8 out_quant[64],
                                                           const guint8 quant[64]);
 
-GST_EXPORT
+GST_CODEC_PARSERS_API
 void    gst_h264_quant_matrix_8x8_get_raster_from_zigzag (guint8 out_quant[64],
                                                           const guint8 quant[64]);
 
-GST_EXPORT
+GST_CODEC_PARSERS_API
 void    gst_h264_quant_matrix_4x4_get_zigzag_from_raster (guint8 out_quant[16],
                                                           const guint8 quant[16]);
 
-GST_EXPORT
+GST_CODEC_PARSERS_API
 void    gst_h264_quant_matrix_4x4_get_raster_from_zigzag (guint8 out_quant[16],
                                                           const guint8 quant[16]);
 
-GST_EXPORT
+GST_CODEC_PARSERS_API
 void gst_h264_video_calculate_framerate (const GstH264SPS * sps, guint field_pic_flag,
     guint pic_struct, gint * fps_num, gint * fps_den);
 
