@@ -28,7 +28,11 @@
 #include <gst/gst.h>
 
 #ifndef GST_WEBRTC_API
-#define GST_WEBRTC_API GST_EXPORT
+# ifdef BUILDING_GST_WEBRTC
+#  define GST_WEBRTC_API GST_API_EXPORT         /* from config.h */
+# else
+#  define GST_WEBRTC_API GST_API_IMPORT
+# endif
 #endif
 
 #include <gst/webrtc/webrtc-enumtypes.h>
@@ -148,7 +152,7 @@ typedef enum /*< underscore_name=gst_webrtc_peer_connection_state >*/
 } GstWebRTCPeerConnectionState;
 
 /**
- * GstWebRTCIceRole:
+ * GstWebRTCICERole:
  * GST_WEBRTC_ICE_ROLE_CONTROLLED: controlled
  * GST_WEBRTC_ICE_ROLE_CONTROLLING: controlling
  */
@@ -156,10 +160,10 @@ typedef enum /*< underscore_name=gst_webrtc_ice_role >*/
 {
   GST_WEBRTC_ICE_ROLE_CONTROLLED,
   GST_WEBRTC_ICE_ROLE_CONTROLLING,
-} GstWebRTCIceRole;
+} GstWebRTCICERole;
 
 /**
- * GstWebRTCIceComponent:
+ * GstWebRTCICEComponent:
  * GST_WEBRTC_ICE_COMPONENT_RTP,
  * GST_WEBRTC_ICE_COMPONENT_RTCP,
  */
@@ -253,4 +257,86 @@ typedef enum /*< underscore_name=gst_webrtc_stats_type >*/
   GST_WEBRTC_STATS_CERTIFICATE,
 } GstWebRTCStatsType;
 
+/**
+ * GstWebRTCFECType:
+ * GST_WEBRTC_FEC_TYPE_NONE: none
+ * GST_WEBRTC_FEC_TYPE_ULP_RED: ulpfec + red
+ */
+typedef enum /*< underscore_name=gst_webrtc_fec_type >*/
+{
+  GST_WEBRTC_FEC_TYPE_NONE,
+  GST_WEBRTC_FEC_TYPE_ULP_RED,
+} GstWebRTCFECType;
+
+/**
+ * GstWebRTCSCTPTransportState:
+ * GST_WEBRTC_SCTP_TRANSPORT_STATE_NEW: new
+ * GST_WEBRTC_SCTP_TRANSPORT_STATE_CONNECTING: connecting
+ * GST_WEBRTC_SCTP_TRANSPORT_STATE_CONNECTED: connected
+ * GST_WEBRTC_SCTP_TRANSPORT_STATE_CLOSED: closed
+ *
+ * See <ulink url="http://w3c.github.io/webrtc-pc/#dom-rtcsctptransportstate">http://w3c.github.io/webrtc-pc/#dom-rtcsctptransportstate</ulink>
+ */
+typedef enum /*< underscore_name=gst_webrtc_sctp_transport_state >*/
+{
+  GST_WEBRTC_SCTP_TRANSPORT_STATE_NEW,
+  GST_WEBRTC_SCTP_TRANSPORT_STATE_CONNECTING,
+  GST_WEBRTC_SCTP_TRANSPORT_STATE_CONNECTED,
+  GST_WEBRTC_SCTP_TRANSPORT_STATE_CLOSED,
+} GstWebRTCSCTPTransportState;
+
+/**
+ * GstWebRTCPriorityType:
+ * GST_WEBRTC_PRIORITY_TYPE_VERY_LOW: very-low
+ * GST_WEBRTC_PRIORITY_TYPE_LOW: low
+ * GST_WEBRTC_PRIORITY_TYPE_MEDIUM: medium
+ * GST_WEBRTC_PRIORITY_TYPE_HIGH: high
+ *
+ * See <ulink url="http://w3c.github.io/webrtc-pc/#dom-rtcprioritytype">http://w3c.github.io/webrtc-pc/#dom-rtcprioritytype</ulink>
+ */
+typedef enum /*< underscore_name=gst_webrtc_priority_type >*/
+{
+  GST_WEBRTC_PRIORITY_TYPE_VERY_LOW = 1,
+  GST_WEBRTC_PRIORITY_TYPE_LOW,
+  GST_WEBRTC_PRIORITY_TYPE_MEDIUM,
+  GST_WEBRTC_PRIORITY_TYPE_HIGH,
+} GstWebRTCPriorityType;
+
+/**
+ * GstWebRTCDataChannelState:
+ * GST_WEBRTC_DATA_CHANNEL_STATE_NEW: new
+ * GST_WEBRTC_DATA_CHANNEL_STATE_CONNECTING: connection
+ * GST_WEBRTC_DATA_CHANNEL_STATE_OPEN: open
+ * GST_WEBRTC_DATA_CHANNEL_STATE_CLOSING: closing
+ * GST_WEBRTC_DATA_CHANNEL_STATE_CLOSED: closed
+ *
+ * See <ulink url="http://w3c.github.io/webrtc-pc/#dom-rtcdatachannelstate">http://w3c.github.io/webrtc-pc/#dom-rtcdatachannelstate</ulink>
+ */
+typedef enum /*< underscore_name=gst_webrtc_data_channel_state >*/
+{
+  GST_WEBRTC_DATA_CHANNEL_STATE_NEW,
+  GST_WEBRTC_DATA_CHANNEL_STATE_CONNECTING,
+  GST_WEBRTC_DATA_CHANNEL_STATE_OPEN,
+  GST_WEBRTC_DATA_CHANNEL_STATE_CLOSING,
+  GST_WEBRTC_DATA_CHANNEL_STATE_CLOSED,
+} GstWebRTCDataChannelState;
+
+/**
+ * GstWebRTCBundlePolicy:
+ * GST_WEBRTC_BUNDLE_POLICY_NONE: none
+ * GST_WEBRTC_BUNDLE_POLICY_BALANCED: balanced
+ * GST_WEBRTC_BUNDLE_POLICY_MAX_COMPAT: max-compat
+ * GST_WEBRTC_BUNDLE_POLICY_MAX_BUNDLE: max-bundle
+ *
+ * See https://tools.ietf.org/html/draft-ietf-rtcweb-jsep-24#section-4.1.1
+ * for more information.
+ */
+typedef enum /*<underscore_name=gst_webrtc_bundle_policy>*/
+{
+  GST_WEBRTC_BUNDLE_POLICY_NONE,
+  GST_WEBRTC_BUNDLE_POLICY_BALANCED,
+  GST_WEBRTC_BUNDLE_POLICY_MAX_COMPAT,
+  GST_WEBRTC_BUNDLE_POLICY_MAX_BUNDLE,
+} GstWebRTCBundlePolicy;
+
 #endif /* __GST_WEBRTC_FWD_H__ */
