@@ -76,6 +76,11 @@ struct _GstH265Parse
   gboolean have_sps;
   gboolean have_pps;
 
+  /* per frame vps/sps/pps check for periodic push codec decision */
+  gboolean have_vps_in_frame;
+  gboolean have_sps_in_frame;
+  gboolean have_pps_in_frame;
+
   /* collected SPS and PPS NALUs */
   GstBuffer *vps_nals[GST_H265_MAX_VPS_COUNT];
   GstBuffer *sps_nals[GST_H265_MAX_SPS_COUNT];
