@@ -51,13 +51,25 @@ struct _GstDecklinkVideoSink
 
   GstDecklinkModeEnum mode;
   gint device_number;
+  GstDecklinkVideoFormat video_format;
+  BMDTimecodeFormat timecode_format;
+  BMDKeyerMode keyer_mode;
+  gint keyer_level;
 
   GstVideoInfo info;
 
   GstClockTime internal_base_time;
   GstClockTime external_base_time;
 
+  /* really an internal start time */
+  GstClockTime internal_time_offset;
+
   GstDecklinkOutput *output;
+
+  GstVideoVBIEncoder *vbiencoder;
+  GstVideoFormat anc_vformat;
+  gint caption_line;
+  guint16 cdp_hdr_sequence_cntr;
 };
 
 struct _GstDecklinkVideoSinkClass
@@ -67,6 +79,9 @@ struct _GstDecklinkVideoSinkClass
 
 GType gst_decklink_video_sink_get_type (void);
 
+void gst_decklink_video_sink_convert_to_internal_clock (GstDecklinkVideoSink * self,
+    GstClockTime * timestamp, GstClockTime * duration);
+
 G_END_DECLS
 
 #endif /* __GST_DECKLINK_VIDEO_SINK_H__ */
