@@ -33,35 +33,42 @@
 #include "gstmsdkbufferpool.h"
 #include "gstmsdksystemmemory.h"
 #include "gstmsdkvideomemory.h"
+#ifndef _WIN32
+#include "gstmsdkallocator_libva.h"
+#endif
 
 GST_DEBUG_CATEGORY_STATIC (gst_debug_msdkbufferpool);
 #define GST_CAT_DEFAULT gst_debug_msdkbufferpool
 
-#define GST_MSDK_BUFFER_POOL_GET_PRIVATE(obj) \
-  (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GST_TYPE_MSDK_BUFFER_POOL, \
-      GstMsdkBufferPoolPrivate))
-
-#define gst_msdk_buffer_pool_parent_class parent_class
-G_DEFINE_TYPE_WITH_CODE (GstMsdkBufferPool, gst_msdk_buffer_pool,
-    GST_TYPE_VIDEO_BUFFER_POOL,
-    GST_DEBUG_CATEGORY_INIT (gst_debug_msdkbufferpool, "msdkbufferpool", 0,
-        "MSDK Buffer Pool"));
+typedef enum _GstMsdkMemoryType
+{
+  GST_MSDK_MEMORY_TYPE_SYSTEM,
+  GST_MSDK_MEMORY_TYPE_VIDEO,
+  GST_MSDK_MEMORY_TYPE_DMABUF,
+} GstMsdkMemoryType;
 
 struct _GstMsdkBufferPoolPrivate
 {
   GstMsdkContext *context;
   GstAllocator *allocator;
   mfxFrameAllocResponse *alloc_response;
-  gboolean use_video_memory;
+  GstMsdkMemoryType memory_type;
   gboolean add_videometa;
 };
 
+#define gst_msdk_buffer_pool_parent_class parent_class
+G_DEFINE_TYPE_WITH_CODE (GstMsdkBufferPool, gst_msdk_buffer_pool,
+    GST_TYPE_VIDEO_BUFFER_POOL, G_ADD_PRIVATE (GstMsdkBufferPool)
+    GST_DEBUG_CATEGORY_INIT (gst_debug_msdkbufferpool, "msdkbufferpool", 0,
+        "MSDK Buffer Pool"));
+
 static const gchar **
 gst_msdk_buffer_pool_get_options (GstBufferPool * pool)
 {
   static const gchar *options[] = { GST_BUFFER_POOL_OPTION_VIDEO_META,
     GST_BUFFER_POOL_OPTION_VIDEO_ALIGNMENT,
     GST_BUFFER_POOL_OPTION_MSDK_USE_VIDEO_MEMORY,
+    GST_BUFFER_POOL_OPTION_MSDK_USE_DMABUF,
     NULL
   };
 
@@ -93,8 +100,9 @@ gst_msdk_buffer_pool_set_config (GstBufferPool * pool, GstStructure * config)
 
   if (allocator
       && (g_strcmp0 (allocator->mem_type, GST_MSDK_SYSTEM_MEMORY_NAME) != 0
+          && g_strcmp0 (allocator->mem_type, GST_MSDK_VIDEO_MEMORY_NAME) != 0
           && g_strcmp0 (allocator->mem_type,
-              GST_MSDK_VIDEO_MEMORY_NAME) != 0)) {
+              GST_MSDK_DMABUF_MEMORY_NAME) != 0)) {
     GST_INFO_OBJECT (pool,
         "This is not MSDK allocator. So this will be ignored");
     gst_object_unref (allocator);
@@ -113,20 +121,37 @@ gst_msdk_buffer_pool_set_config (GstBufferPool * pool, GstStructure * config)
     gst_buffer_pool_config_set_video_alignment (config, &alignment);
   }
 
-  priv->use_video_memory = gst_buffer_pool_config_has_option (config,
-      GST_BUFFER_POOL_OPTION_MSDK_USE_VIDEO_MEMORY);
+  if (gst_buffer_pool_config_has_option (config,
+          GST_BUFFER_POOL_OPTION_MSDK_USE_VIDEO_MEMORY))
+    priv->memory_type = GST_MSDK_MEMORY_TYPE_VIDEO;
 
-  if (priv->use_video_memory && (!priv->context || !priv->alloc_response)) {
+  if ((priv->memory_type & GST_MSDK_MEMORY_TYPE_VIDEO) && (!priv->context
+          || !priv->alloc_response)) {
     GST_ERROR_OBJECT (pool,
         "No MSDK context or Allocation response for using video memory");
     goto error_invalid_config;
   }
 
+  if (gst_buffer_pool_config_has_option (config,
+          GST_BUFFER_POOL_OPTION_MSDK_USE_DMABUF))
+    priv->memory_type |= GST_MSDK_MEMORY_TYPE_DMABUF;
+
+  if (!(priv->memory_type & GST_MSDK_MEMORY_TYPE_VIDEO) &&
+      priv->memory_type & GST_MSDK_MEMORY_TYPE_DMABUF) {
+    GST_WARNING_OBJECT (pool,
+        "Can't use dmabuf since this is system msdk bufferpool");
+    priv->memory_type = GST_MSDK_MEMORY_TYPE_SYSTEM;
+  }
+
   /* create a new allocator if needed */
   if (!allocator) {
     GstAllocationParams params = { 0, 31, 0, 0, };
 
-    if (priv->use_video_memory)
+    if (priv->memory_type & GST_MSDK_MEMORY_TYPE_DMABUF)
+      allocator =
+          gst_msdk_dmabuf_allocator_new (priv->context, &video_info,
+          priv->alloc_response);
+    else if (priv->memory_type & GST_MSDK_MEMORY_TYPE_VIDEO)
       allocator =
           gst_msdk_video_allocator_new (priv->context, &video_info,
           priv->alloc_response);
@@ -187,7 +212,9 @@ gst_msdk_buffer_pool_alloc_buffer (GstBufferPool * pool,
 
   buf = gst_buffer_new ();
 
-  if (priv->use_video_memory)
+  if (priv->memory_type & GST_MSDK_MEMORY_TYPE_DMABUF)
+    mem = gst_msdk_dmabuf_memory_new (priv->allocator);
+  else if (priv->memory_type & GST_MSDK_MEMORY_TYPE_VIDEO)
     mem = gst_msdk_video_memory_new (priv->allocator);
   else
     mem = gst_msdk_system_memory_new (priv->allocator);
@@ -201,7 +228,9 @@ gst_msdk_buffer_pool_alloc_buffer (GstBufferPool * pool,
     GstVideoMeta *vmeta;
     GstVideoInfo *info;
 
-    if (priv->use_video_memory)
+    if (priv->memory_type & GST_MSDK_MEMORY_TYPE_DMABUF)
+      info = &GST_MSDK_DMABUF_ALLOCATOR_CAST (priv->allocator)->image_info;
+    else if (priv->memory_type & GST_MSDK_MEMORY_TYPE_VIDEO)
       info = &GST_MSDK_VIDEO_ALLOCATOR_CAST (priv->allocator)->image_info;
     else
       info = &GST_MSDK_SYSTEM_ALLOCATOR_CAST (priv->allocator)->image_info;
@@ -211,7 +240,8 @@ gst_msdk_buffer_pool_alloc_buffer (GstBufferPool * pool,
         GST_VIDEO_INFO_WIDTH (info), GST_VIDEO_INFO_HEIGHT (info),
         GST_VIDEO_INFO_N_PLANES (info), info->offset, info->stride);
 
-    if (priv->use_video_memory) {
+    if ((priv->memory_type & GST_MSDK_MEMORY_TYPE_VIDEO) &&
+        !(priv->memory_type & GST_MSDK_MEMORY_TYPE_DMABUF)) {
       vmeta->map = gst_video_meta_map_msdk_memory;
       vmeta->unmap = gst_video_meta_unmap_msdk_memory;
     }
@@ -236,6 +266,7 @@ gst_msdk_buffer_pool_acquire_buffer (GstBufferPool * pool,
   GstBuffer *buf = NULL;
   GstFlowReturn ret;
   mfxFrameSurface1 *surface;
+  gint fd;
 
   ret =
       GST_BUFFER_POOL_CLASS (parent_class)->acquire_buffer (pool, &buf, params);
@@ -245,7 +276,7 @@ gst_msdk_buffer_pool_acquire_buffer (GstBufferPool * pool,
    * So we need to confirm if it's unlocked every time a gst buffer is acquired.
    * If it's still locked, we can replace it with new unlocked/unused surface.
    */
-  if (ret != GST_FLOW_OK || !priv->use_video_memory) {
+  if (ret != GST_FLOW_OK || priv->memory_type == GST_MSDK_MEMORY_TYPE_SYSTEM) {
     if (buf)
       *out_buffer_ptr = buf;
     return ret;
@@ -253,12 +284,29 @@ gst_msdk_buffer_pool_acquire_buffer (GstBufferPool * pool,
 
   surface = gst_msdk_get_surface_from_buffer (buf);
   if (!surface || surface->Data.Locked > 0) {
-    if (!gst_msdk_video_memory_get_surface_available (GST_MSDK_VIDEO_MEMORY_CAST
-            (gst_buffer_peek_memory (buf, 0)))) {
+    if (!gst_msdk_video_memory_get_surface_available (gst_buffer_peek_memory
+            (buf, 0))) {
       GST_WARNING_OBJECT (pool, "failed to get new surface available");
       return GST_FLOW_ERROR;
     }
   }
+#ifndef _WIN32
+  /* When using dmabuf, we should confirm that the fd of memeory and
+   * the fd of surface match, since there is no guarantee that fd matches
+   * between surface and memory.
+   */
+  if (priv->memory_type & GST_MSDK_MEMORY_TYPE_DMABUF) {
+    surface = gst_msdk_get_surface_from_buffer (buf);
+    gst_msdk_get_dmabuf_info_from_surface (surface, &fd, NULL);
+
+    if (gst_dmabuf_memory_get_fd (gst_buffer_peek_memory (buf, 0)) != fd) {
+      GstMemory *mem;
+      mem = gst_msdk_dmabuf_memory_new_with_surface (priv->allocator, surface);
+      gst_buffer_replace_memory (buf, 0, mem);
+      gst_buffer_unset_flags (buf, GST_BUFFER_FLAG_TAG_MEMORY);
+    }
+  }
+#endif
 
   *out_buffer_ptr = buf;
   return GST_FLOW_OK;
@@ -271,18 +319,17 @@ gst_msdk_buffer_pool_release_buffer (GstBufferPool * pool, GstBuffer * buf)
   GstMsdkBufferPool *msdk_pool = GST_MSDK_BUFFER_POOL_CAST (pool);
   GstMsdkBufferPoolPrivate *priv = msdk_pool->priv;
 
-  if (!priv->use_video_memory)
+  if (priv->memory_type == GST_MSDK_MEMORY_TYPE_SYSTEM)
     goto done;
 
   surface = gst_msdk_get_surface_from_buffer (buf);
   if (!surface)
     goto done;
 
-  gst_msdk_video_memory_release_surface (GST_MSDK_VIDEO_MEMORY_CAST
-      (gst_buffer_peek_memory (buf, 0)));
+  gst_msdk_video_memory_release_surface (gst_buffer_peek_memory (buf, 0));
 
 done:
-  return GST_BUFFER_POOL_CLASS (parent_class)->release_buffer (pool, buf);
+  GST_BUFFER_POOL_CLASS (parent_class)->release_buffer (pool, buf);
 }
 
 static void
@@ -300,7 +347,7 @@ gst_msdk_buffer_pool_finalize (GObject * object)
 static void
 gst_msdk_buffer_pool_init (GstMsdkBufferPool * pool)
 {
-  pool->priv = GST_MSDK_BUFFER_POOL_GET_PRIVATE (pool);
+  pool->priv = gst_msdk_buffer_pool_get_instance_private (pool);
 }
 
 static void
@@ -309,8 +356,6 @@ gst_msdk_buffer_pool_class_init (GstMsdkBufferPoolClass * klass)
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GstBufferPoolClass *pool_class = GST_BUFFER_POOL_CLASS (klass);
 
-  g_type_class_add_private (klass, sizeof (GstMsdkBufferPoolPrivate));
-
   object_class->finalize = gst_msdk_buffer_pool_finalize;
 
   pool_class->get_options = gst_msdk_buffer_pool_get_options;
