@@ -33,24 +33,34 @@
 #  include <config.h>
 #endif
 
-#ifdef HAVE_LIBMFX
-#  include <mfx/mfxplugin.h>
-#else
-#  include "mfxplugin.h"
-#endif
+#include <mfxplugin.h>
 
 #include "gstmsdkh265dec.h"
+#include "gstmsdkvideomemory.h"
 
 GST_DEBUG_CATEGORY_EXTERN (gst_msdkh265dec_debug);
 #define GST_CAT_DEFAULT gst_msdkh265dec_debug
 
+/* TODO: update both sink and src dynamically */
 static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("video/x-h265, "
         "width = (int) [ 1, MAX ], height = (int) [ 1, MAX ], "
         "stream-format = (string) byte-stream , alignment = (string) au , "
-        "profile = (string) main ")
+        "profile = (string) { main, main-10 } ")
+    );
+
+static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
+    GST_PAD_SRC,
+    GST_PAD_ALWAYS,
+    GST_STATIC_CAPS ("video/x-raw, "
+        "format = (string) { NV12, P010_10LE }, "
+        "framerate = (fraction) [0, MAX], "
+        "width = (int) [ 16, MAX ], height = (int) [ 16, MAX ],"
+        "interlace-mode = (string) progressive;"
+        GST_VIDEO_CAPS_MAKE_WITH_FEATURES (GST_CAPS_FEATURE_MEMORY_DMABUF,
+            "{ NV12 }") ";")
     );
 
 #define gst_msdkh265dec_parent_class parent_class
@@ -82,19 +92,73 @@ gst_msdkh265dec_configure (GstMsdkDec * decoder)
   }
 
   decoder->param.mfx.CodecId = MFX_CODEC_HEVC;
-  decoder->param.mfx.CodecProfile = MFX_PROFILE_HEVC_MAIN;
+
+  /* This is a deprecated attribute in msdk-2017 version, but some
+   * customers still using this for low-latency streaming of non-b-frame
+   * encoded streams */
+  decoder->param.mfx.DecodedOrder = h265dec->output_order;
   return TRUE;
 }
 
+static void
+gst_msdkdec_h265_set_property (GObject * object, guint prop_id,
+    const GValue * value, GParamSpec * pspec)
+{
+  GstMsdkH265Dec *thiz = GST_MSDKH265DEC (object);
+  GstState state;
+
+  GST_OBJECT_LOCK (thiz);
+  state = GST_STATE (thiz);
+
+  if (!gst_msdkdec_prop_check_state (state, pspec)) {
+    GST_WARNING_OBJECT (thiz, "setting property in wrong state");
+    GST_OBJECT_UNLOCK (thiz);
+    return;
+  }
+  switch (prop_id) {
+    case GST_MSDKDEC_PROP_OUTPUT_ORDER:
+      thiz->output_order = g_value_get_enum (value);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+  }
+  GST_OBJECT_UNLOCK (thiz);
+  return;
+}
+
+static void
+gst_msdkdec_h265_get_property (GObject * object, guint prop_id, GValue * value,
+    GParamSpec * pspec)
+{
+  GstMsdkH265Dec *thiz = GST_MSDKH265DEC (object);
+
+  GST_OBJECT_LOCK (thiz);
+  switch (prop_id) {
+    case GST_MSDKDEC_PROP_OUTPUT_ORDER:
+      g_value_set_enum (value, thiz->output_order);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+  }
+  GST_OBJECT_UNLOCK (thiz);
+}
+
 static void
 gst_msdkh265dec_class_init (GstMsdkH265DecClass * klass)
 {
+  GObjectClass *gobject_class;
   GstElementClass *element_class;
   GstMsdkDecClass *decoder_class;
 
+  gobject_class = G_OBJECT_CLASS (klass);
   element_class = GST_ELEMENT_CLASS (klass);
   decoder_class = GST_MSDKDEC_CLASS (klass);
 
+  gobject_class->set_property = gst_msdkdec_h265_set_property;
+  gobject_class->get_property = gst_msdkdec_h265_get_property;
+
   decoder_class->configure = GST_DEBUG_FUNCPTR (gst_msdkh265dec_configure);
 
   gst_element_class_set_static_metadata (element_class,
@@ -103,10 +167,14 @@ gst_msdkh265dec_class_init (GstMsdkH265DecClass * klass)
       "H265 video decoder based on Intel Media SDK",
       "Scott D Phillips <scott.d.phillips@intel.com>");
 
+  gst_msdkdec_prop_install_output_oder_property (gobject_class);
+
   gst_element_class_add_static_pad_template (element_class, &sink_factory);
+  gst_element_class_add_static_pad_template (element_class, &src_factory);
 }
 
 static void
 gst_msdkh265dec_init (GstMsdkH265Dec * thiz)
 {
+  thiz->output_order = PROP_OUTPUT_ORDER_DEFAULT;
 }
