@@ -27,18 +27,42 @@
 
 #include <string.h>
 
-#if HAVE_GST_GL
+GST_DEBUG_CATEGORY_STATIC (gst_nv_h264_enc_debug);
+#define GST_CAT_DEFAULT gst_nv_h264_enc_debug
+
+#if HAVE_NVENC_GST_GL
 #include <cuda.h>
 #include <cuda_runtime_api.h>
 #include <cuda_gl_interop.h>
-#define GST_USE_UNSTABLE_API
 #include <gst/gl/gl.h>
 #endif
 
 #define parent_class gst_nv_h264_enc_parent_class
 G_DEFINE_TYPE (GstNvH264Enc, gst_nv_h264_enc, GST_TYPE_NV_BASE_ENC);
 
+#if HAVE_NVENC_GST_GL
+#define GL_CAPS_STR \
+  ";" \
+  "video/x-raw(memory:GLMemory), " \
+  "format = (string) { NV12, Y444 }, " \
+  "width = (int) [ 16, 4096 ], height = (int) [ 16, 2160 ], " \
+  "framerate = (fraction) [0, MAX]," \
+  "interlace-mode = { progressive, mixed, interleaved } "
+#else
+#define GL_CAPS_STR ""
+#endif
+
 /* *INDENT-OFF* */
+static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
+    GST_PAD_SINK,
+    GST_PAD_ALWAYS,
+    GST_STATIC_CAPS ("video/x-raw, " "format = (string) { NV12, I420 }, "       // TODO: YV12, Y444 support
+        "width = (int) [ 16, 4096 ], height = (int) [ 16, 2160 ], "
+        "framerate = (fraction) [0, MAX],"
+        "interlace-mode = { progressive, mixed, interleaved } "
+        GL_CAPS_STR
+    ));
+
 static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
@@ -89,6 +113,7 @@ gst_nv_h264_enc_class_init (GstNvH264EncClass * klass)
   nvenc_class->set_src_caps = gst_nv_h264_enc_set_src_caps;
   nvenc_class->set_pic_params = gst_nv_h264_enc_set_pic_params;
 
+  gst_element_class_add_static_pad_template (element_class, &sink_factory);
   gst_element_class_add_static_pad_template (element_class, &src_factory);
 
   gst_element_class_set_static_metadata (element_class,
@@ -97,6 +122,9 @@ gst_nv_h264_enc_class_init (GstNvH264EncClass * klass)
       "Encode H.264 video streams using NVIDIA's hardware-accelerated NVENC encoder API",
       "Tim-Philipp Müller <tim@centricular.com>\n"
       "Matthew Waters <matthew@centricular.com>");
+
+  GST_DEBUG_CATEGORY_INIT (gst_nv_h264_enc_debug,
+      "nvh264enc", 0, "Nvidia H.264 encoder");
 }
 
 static void
@@ -119,9 +147,6 @@ _get_supported_profiles (GstNvH264Enc * nvenc)
   GValue val = G_VALUE_INIT;
   guint i, n, n_profiles;
 
-  if (nvenc->supported_profiles)
-    return TRUE;
-
   nv_ret =
       NvEncGetEncodeProfileGUIDCount (GST_NV_BASE_ENC (nvenc)->encoder,
       NV_ENC_CODEC_H264_GUID, &n);
@@ -164,8 +189,8 @@ _get_supported_profiles (GstNvH264Enc * nvenc)
     return FALSE;
 
   GST_OBJECT_LOCK (nvenc);
-  nvenc->supported_profiles = g_new0 (GValue, 1);
-  *nvenc->supported_profiles = list;
+  g_free (nvenc->supported_profiles);
+  nvenc->supported_profiles = g_memdup (&list, sizeof (GValue));
   GST_OBJECT_UNLOCK (nvenc);
 
   return TRUE;
@@ -214,8 +239,6 @@ gst_nv_h264_enc_close (GstVideoEncoder * enc)
   GstNvH264Enc *nvenc = GST_NV_H264_ENC (enc);
 
   GST_OBJECT_LOCK (nvenc);
-  if (nvenc->supported_profiles)
-    g_value_unset (nvenc->supported_profiles);
   g_free (nvenc->supported_profiles);
   nvenc->supported_profiles = NULL;
   GST_OBJECT_UNLOCK (nvenc);
@@ -249,7 +272,6 @@ _get_interlace_modes (GstNvH264Enc * nvenc)
     gst_value_list_append_value (list, &val);
     g_value_set_static_string (&val, "mixed");
     gst_value_list_append_value (list, &val);
-    g_value_unset (&val);
   }
   /* TODO: figure out what nvenc frame based interlacing means in gst terms */
 
@@ -275,7 +297,6 @@ gst_nv_h264_enc_getcaps (GstVideoEncoder * enc, GstCaps * filter)
 
     val = _get_interlace_modes (nvenc);
     gst_caps_set_value (supported_incaps, "interlace-mode", val);
-    g_value_unset (val);
     g_free (val);
 
     GST_LOG_OBJECT (enc, "codec input caps %" GST_PTR_FORMAT, supported_incaps);
