@@ -533,7 +533,7 @@ out:
 
 gboolean
 gst_wasapi_util_get_device_client (GstElement * self,
-    gboolean capture, gint role, const wchar_t * device_strid,
+    gint data_flow, gint role, const wchar_t * device_strid,
     IMMDevice ** ret_device, IAudioClient ** ret_client)
 {
   gboolean res = FALSE;
@@ -546,8 +546,8 @@ gst_wasapi_util_get_device_client (GstElement * self,
     goto beach;
 
   if (!device_strid) {
-    hr = IMMDeviceEnumerator_GetDefaultAudioEndpoint (enumerator,
-        capture ? eCapture : eRender, role, &device);
+    hr = IMMDeviceEnumerator_GetDefaultAudioEndpoint (enumerator, data_flow,
+        role, &device);
     HR_FAILED_GOTO (hr, IMMDeviceEnumerator::GetDefaultAudioEndpoint, beach);
   } else {
     hr = IMMDeviceEnumerator_GetDevice (enumerator, device_strid, &device);
@@ -695,7 +695,7 @@ static guint64
 gst_wasapi_util_waveformatex_to_channel_mask (WAVEFORMATEXTENSIBLE * format,
     GstAudioChannelPosition ** out_position)
 {
-  int ii, ch;
+  int ii;
   guint64 mask = 0;
   WORD nChannels = format->Format.nChannels;
   DWORD dwChannelMask = format->dwChannelMask;
@@ -719,19 +719,14 @@ gst_wasapi_util_waveformatex_to_channel_mask (WAVEFORMATEXTENSIBLE * format,
 
   /* Map WASAPI's channel mask to Gstreamer's channel mask and positions.
    * If the no. of bits in the mask > nChannels, we will ignore the extra. */
-  for (ii = 0, ch = 0; ii < G_N_ELEMENTS (wasapi_to_gst_pos) && ch < nChannels;
-      ii++) {
+  for (ii = 0; ii < nChannels; ii++) {
     if (!(dwChannelMask & wasapi_to_gst_pos[ii].wasapi_pos))
-      /* no match, try next */
+      /* Non-positional or unknown position, warn? */
       continue;
     mask |= G_GUINT64_CONSTANT (1) << wasapi_to_gst_pos[ii].gst_pos;
-    pos[ch++] = wasapi_to_gst_pos[ii].gst_pos;
+    pos[ii] = wasapi_to_gst_pos[ii].gst_pos;
   }
 
-  /* XXX: Warn if some channel masks couldn't be mapped? */
-
-  GST_DEBUG ("Converted WASAPI mask 0x%x -> 0x%x", dwChannelMask, mask);
-
 out:
   if (out_position)
     *out_position = pos;
@@ -778,12 +773,8 @@ gst_wasapi_util_parse_waveformatex (WAVEFORMATEXTENSIBLE * format,
     gst_structure_set (s,
         "format", G_TYPE_STRING, afmt,
         "channels", G_TYPE_INT, format->Format.nChannels,
-        "rate", G_TYPE_INT, format->Format.nSamplesPerSec, NULL);
-
-    if (channel_mask) {
-      gst_structure_set (s,
-          "channel-mask", GST_TYPE_BITMASK, channel_mask, NULL);
-    }
+        "rate", G_TYPE_INT, format->Format.nSamplesPerSec,
+        "channel-mask", GST_TYPE_BITMASK, channel_mask, NULL);
   }
 
   return TRUE;
@@ -839,11 +830,11 @@ gboolean
 gst_wasapi_util_initialize_audioclient (GstElement * self,
     GstAudioRingBufferSpec * spec, IAudioClient * client,
     WAVEFORMATEX * format, guint sharemode, gboolean low_latency,
-    guint * ret_devicep_frames)
+    gboolean loopback, guint * ret_devicep_frames)
 {
   REFERENCE_TIME default_period, min_period;
   REFERENCE_TIME device_period, device_buffer_duration;
-  guint rate;
+  guint rate, stream_flags;
   guint32 n_frames;
   HRESULT hr;
 
@@ -870,8 +861,12 @@ gst_wasapi_util_initialize_audioclient (GstElement * self,
         min_period, &device_period, &device_buffer_duration);
   }
 
-  hr = IAudioClient_Initialize (client, sharemode,
-      AUDCLNT_STREAMFLAGS_EVENTCALLBACK, device_buffer_duration,
+  stream_flags = AUDCLNT_STREAMFLAGS_EVENTCALLBACK;
+  if (loopback)
+    stream_flags |= AUDCLNT_STREAMFLAGS_LOOPBACK;
+
+  hr = IAudioClient_Initialize (client, sharemode, stream_flags,
+      device_buffer_duration,
       /* This must always be 0 in shared mode */
       sharemode == AUDCLNT_SHAREMODE_SHARED ? 0 : device_period, format, NULL);
 
@@ -889,9 +884,8 @@ gst_wasapi_util_initialize_audioclient (GstElement * self,
     GST_WARNING_OBJECT (self, "trying to re-initialize with period %i "
         "(%i frames, %i rate)", (int) device_period, n_frames, rate);
 
-    hr = IAudioClient_Initialize (client, sharemode,
-        AUDCLNT_STREAMFLAGS_EVENTCALLBACK, device_period,
-        device_period, format, NULL);
+    hr = IAudioClient_Initialize (client, sharemode, stream_flags,
+        device_period, device_period, format, NULL);
   }
   HR_FAILED_RET (hr, IAudioClient::Initialize, FALSE);
 
@@ -912,9 +906,11 @@ gst_wasapi_util_initialize_audioclient (GstElement * self,
 gboolean
 gst_wasapi_util_initialize_audioclient3 (GstElement * self,
     GstAudioRingBufferSpec * spec, IAudioClient3 * client,
-    WAVEFORMATEX * format, gboolean low_latency, guint * ret_devicep_frames)
+    WAVEFORMATEX * format, gboolean low_latency, gboolean loopback,
+    guint * ret_devicep_frames)
 {
   HRESULT hr;
+  gint stream_flags;
   guint devicep_frames;
   guint defaultp_frames, fundp_frames, minp_frames, maxp_frames;
   WAVEFORMATEX *tmpf;
@@ -934,8 +930,12 @@ gst_wasapi_util_initialize_audioclient3 (GstElement * self,
      * https://bugzilla.gnome.org/show_bug.cgi?id=794497 */
     devicep_frames = maxp_frames;
 
-  hr = IAudioClient3_InitializeSharedAudioStream (client,
-      AUDCLNT_STREAMFLAGS_EVENTCALLBACK, devicep_frames, format, NULL);
+  stream_flags = AUDCLNT_STREAMFLAGS_EVENTCALLBACK;
+  if (loopback)
+    stream_flags |= AUDCLNT_STREAMFLAGS_LOOPBACK;
+
+  hr = IAudioClient3_InitializeSharedAudioStream (client, stream_flags,
+      devicep_frames, format, NULL);
   HR_FAILED_RET (hr, IAudioClient3::InitializeSharedAudioStream, FALSE);
 
   hr = IAudioClient3_GetCurrentSharedModeEnginePeriod (client, &tmpf,
